/*
 * file: ~/public_html/tmp/new_log/new-log.c
 *
 * Created by Ola Mrtensson, ola@matrix.df.lth.se
 *
 * Time-stamp: <97/02/28 16:36:43 ola>
 *
 *
 * Quick cgi-bin hack for logging accesses to html-pages
 * argv 1: index string
 * will add one to nuber corresponding to index-string
 *
 * "access-file"-format; list of:
 * [^]<index-string>[SPC]<#number>[EOS]
 * ...
 * [EOF]
 *
 * NOTE!!!: <#number> must be "WIDTH" number of chars!!!
 *
 * 
 * Change: 960131    First quick hack
 * Change: 960416    Working and using lock and retries
 *
 */

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <sys/file.h>

#include "gd.h"
#include "gdfontg.h"


#define WIDTH       4                            /* size of picture in chars */
#define HEIGHT      1
#define XOFF        2                                     /* size of borders */
#define YOFF        2
#define CHARHEIGHT 15                                    /* FontGiant = 9x15 */
#define CHARWIDTH   9
#define XSIZE       (((XOFF)*3)+((CHARWIDTH)*(WIDTH))+1)     /* size in pixs */
#define YSIZE       (((YOFF)*2)+((CHARHEIGHT)*(HEIGHT))-1)   /* size in pixs */

#define EOS '\0'

#define COUNTER_LOG "/home/ola/public_html/accesses.log"

#define RETRIES     5               /* number of tries to access COUNTER_LOG */
#define RETRY_SLEEP 2                        /* seconds to wait before retry */


/*              */
/* Main Program */
/*              */
int main(int argc,char *argv[])
{
  char stmp[10],out[4];
  char t1[80];
  char t2[80];
  int value,zeros;
  int t,i;
  FILE *f;

  gdImagePtr im;                                            /* the image     */
  FILE *pic;                                                /* picture file  */
  int bg,pen;                                               /* color indexes */
  
  if (argc!=2) {
    fprintf(stderr,"new-log: wrong number of arguments.\n");
    return 1;
  }

  /* Try to open the workfile for reading startvalue */
  if ((f = fopen(COUNTER_LOG,"r+"))<0) {
    fprintf(stderr,"new-log: can't find '%s'.\n",COUNTER_LOG);
    return 1;
  }

  for(t=0;t<RETRIES;t++) {
    if (flock(fileno(f), LOCK_EX | LOCK_NB)<0) {
      sleep(RETRY_SLEEP);
    }
    else
      break;
  }

  if (t==RETRIES) {                           /* couldn't access COUNTER_LOG */
    strcpy(out,"BUSY");
  }
  else {
    while((fscanf(f,"%80s %80s",t1,t2)!=EOF)
          && (strcmp(t1,argv[1])!=0))
      ;

    if (strcmp(t1,argv[1])!=0) {
      fprintf(stderr,"new-log: can't find label '%s'\n",argv[1]);
      flock(fileno(f), LOCK_UN);
      fclose(f);
      return 1;
    }
    else {
      value=atoi(t2);
      value++;
      itoa(value,stmp);

      /* put right number of leading '0' in text */
      out[0]=EOS;
      zeros=(WIDTH-strlen(stmp));
      for(i=0;i<zeros;i++)
	strcat(out,"0");
      strcat(out,stmp);

      /* write new value */
      lseek(fileno(f),ftell(f)-WIDTH,SEEK_SET);
      write(fileno(f),out,WIDTH);
    }
  }
  im = gdImageCreate(XSIZE,YSIZE);

  /* allocate color, first declared => color 0 => bg-color */
  bg = gdImageColorAllocate(im, 0, 0, 0);
  pen = gdImageColorAllocate(im, 255, 255, 255);

  /* gdImageColorTransparent(im, bg); */

  /* write border */
  gdImageLine(im, 0, 0, XSIZE-1, 0, pen);              /* top    */
  gdImageLine(im, 0, 0, 0, YSIZE-1, pen);              /* left   */
  gdImageLine(im, 0, YSIZE-1, XSIZE-1, YSIZE-1, pen);  /* bottom */
  gdImageLine(im, XSIZE-1, 0, XSIZE-1, YSIZE-1, pen);  /* right  */

  /* write number */
  gdImageString(im, gdFontGiant, (XOFF+1), YOFF, out, pen);

  /* Open a 'stdout' for writing. "wb" means "write binary", important
     under MSDOS, harmless under Unix. */
  pic = fdopen(1, "wb");
  fprintf(pic,"Content-type: image/gif\n\n");

  gdImageGif(im, pic);                 /* Output the image to the disk file. */
  fclose(pic);
  gdImageDestroy(im);                        /* Destroy the image in memory. */

  flock(fileno(f), LOCK_UN);
  fclose(f);
  return 0;
}
